# FreiCalib

This is the calibration software developed in conjunction with our recent publication

    ```
	@TechReport{Freipose2020,
	  author    = {Christian Zimmermann, Artur Schneider, Ilka Diester and Thomas Brox},
	  title     = {FreiPose: A Framework for Deep Animal Motion Capture from Multiple Views},
	  booktitle    = {TBD},
	  year      = {2020},
	  url          = {"https://lmb.informatik.uni-freiburg.de/projects/freipose/"}
	}
    ```

It allows to calibrate a multi camera rig from dedicated sequences using a calibration object. 
Help to create such a calibration object is provided

    
## Tutorial using Docker

Clone repository OR Download repository:

    git clone https://github.com/zimmerm/FreiCalib
    wget --no-check-certificate https://lmb.informatik.uni-freiburg.de/data/RatTrack/data/FreiCalib-master.zip && unzip FreiCalib-master.zip && mv FreiCalib-master FreiCalib
    

Define path to the FreiCalib folder:

    FC_DIR="${PWD}/FreiCalib/"
    
Download example data:
    
     cd ${FC_DIR}/data && wget --no-check-certificate https://lmb.informatik.uni-freiburg.de/data/RatTrack/data/calib.zip && unzip calib.zip && rm calib.zip

Build docker container (from repo base folder):
    
    cd ${FC_DIR} && make
    
Start docker container:
    
    bash docker_run.sh

Calculate calibration (inside the container):

    cd ~/FreiCalib-master/ && python calib_M.py /host/marker_32h11b2_4x10x_5cm.json /host/run000_cam1.avi
    
Reprojection error before bundling: 52.08 pixel  *
Final reprojection error: 1.10 pixel *
Notes:
    
    - Simply point it to one of the video files. The program will try to match cam0, cam1, etc to find the other files
    - Instead of giving it a video file you can also provide it with a folder, where subfolders called cam0, cam1, cam2 branch off.
    - ...


Check calibration on same sequence (inside the container):

    cd ~/FreiCalib-master/ && python check_M.py /host/marker_32h11b2_4x10x_5cm.json /host/run000_cam1.avi /host/M.json
    
Final reprojection error: 1.84 pixel (Larger than before due to non optimized object poses)

Check calibration on another sequence (inside the container):

    cd ~/FreiCalib-master/ && python check_M.py /host/marker_32h11b2_4x10x_5cm.json /host/run001_cam1.avi /host/M.json

Final reprojection error: 2.40 pixel *


*: exact value will differ due to some randomness
   

## User guide

### Create a new calibration pattern

When creating a new pattern you will need to physically create an object that you move in front of your cameras
while recording a calibration video sequence.  

Example call:

    python create_marker.py --nx 4 --ny 10 --tsize 0.05 --double
    
Which will give you three files in `./tags/`. Two PDF's which are the front and back side of the calibration object and 
a json file containing information to be passed to subsequent scripts in order to work with this specific pattern.
This call will create a double sided (front and back) marker where each cell is 5 cm large and where patterns are arranged
in a 4 by 10 grid. In total there are 2*4*10=80 patterns.
    
### Calibration: Intrinsic

Intrinsic calibration will determine imaging parameters of a single cameras. If you wish to calculate this explicitly you
 can also use this routine in isolation.
 
Example call:

    python calib_K.py $MARKER_PATH $DATA_PATH

Where `$MARKER_PATH` should point to the json file created by create_marker.py and `$DATA_PATH` either points to a directory
 of images or a video file. Supported file types are: 'jpg', 'jpeg', 'png' and 'bmp'
 
If you wish to check an intrinsic calibration and get some idea on how to improve the calibration process two visualization scripts are being provided:

    python vis_det.py $MARKER_PATH $DATA_PATH
    python vis_K.py $MARKER_PATH $DATA_PATH
    
Certain names relative to the image/video data are being assumed while calling these scripts. 
When dealing with intrinsic calibration the assumed names are:

    - Calibration board detections are being stored in detection.json files
    - Intrinsic calibration is stored as K.json
    
But these are changeable via additional arguments while calling the scripts. 
All scripts provide help on their usage when being called with the `--help` argument.   
 
### Calibration: Extrinsic
 
Extrinsic calibration will determine the relative position and orientation of the cameras with respect to each other.
 
Example call:

    python calib_M.py $MARKER_PATH $DATA_PATH
    
This script will dump:

    - Two files per camera
    - For each camera one detections_cam%d.json and one K_cam%d.json
    - One M.json containing the extrinsic calibration
    
To check the calibration the following script can be used: 

    python check_M.py $MARKER_PATH $DATA_PATH $CALIB_PATH

Where `$MARKER_PATH` should point to the json file created by create_marker.py and `$DATA_PATH` either points to a directory
 of images or a video file. Supported file types are: 'jpg', 'jpeg', 'png' and 'bmp'. `$CALIB_PATH` is the M.json to be used.


## Manual installation (not recommended)

### Build Apriltag Detector

    cd TagDetector && python setupBatch.py build_ext --inplace
    rm -r TagDetector/build && rm TagDetector/AprilTagDetectorBatch.cp*


### Build Bundle

Install ceres:

    http://ceres-solver.org/installation.html
    
Build this package:

    cd Bundle && mkdir build && cd build && cmake .. && make
    
What I have to do:
    
    export Ceres_DIR="/misc/lmbraid18/zimmermc/libs/ceres-solver-1.14.0-install/lib/cmake/Ceres/"
    
## TODO:
    
    - Add a short description for each callable script.
    - Make test data downloadable, 
    - Add good practices to get good calibration results: Good coverage of normals, distances and the viewing plane
    - How to create a Calibration pattern
    - Replace wget of the old repository by git clone when this repo goes online.
    - Also output distributions from calib_M
    - Make Bundle also optimize the front/back offset (Keep front and back model points separate and introduce offset as variable, that is strongly regularized)